#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2014-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
=end
require 'singleton'
require 'date'
require 'digest'
require_relative 'logging'

=begin
	An object of this type represents one unfinished or queued attempt
	to download a file.
=end
class Download
	self.extend(Logging)
	@@log = init_logger(STDOUT)
	@@log.level = $log_level
	@@SHA256 = Digest::SHA256.new

	class DownloadException < StandardError
	end

	# set attributes to reasonable defaults
	def initialize(url, local = nil)
		@log = @@log
		@url = url	
		@local_file = local
		@date = Date.today
		@updated = DateTime.now
		@position = 0
		@size = 0
		@digest = ""
	end

	def to_s
		s = self.class.name
		s << '{'
		instance_variables.each do |v|
			unless [:@target, :@log].include?(v)
				val = instance_variable_get(v)
				s << v.to_s << '=' << (val ? val.to_s : 'NIL')
				s << ' [' << val.class.name  << ']' if val
				s << ', '
			end
		end
		s << '}'
	end

	# define the elements that will be saved in the base of queued
	# downloads
	def marshal_dump
		[@date, @updated, @url, @local_file, @position, @size, @digest]
	end

	# reconstruct a Download-object from the previously stored
	# dump
	def marshal_load array
		@date, @updated, @url, @local_file, @position, @size, @digest = array
		if(array.include?(nil) )
			raise DownloadException.new('Incomplete Download-data. NIL-values detected')
		end
		if(@local_file && File.exist?(@local_file))
			if(File.readable?(@local_file))

				d = @@SHA256.hexdigest(File::read(@local_file))
				if(d && d != @digest)
					@digest ||= " N I L "
					de = DownloadException.new('Calculated digest for current file is different from stored value' << "\n\t" << d << " != " << @digest << "\n" )
					puts(de.message)
					puts "Do you want to continue anyway (Y/n)?"
					r = wait_for_user.chr
					if('Y' != r.upcase)
						raise de
						exit true
					end
				end
			else
				puts ('local file ' << @local_file << ' is unusable! Aborting.')
				exit false

			end
		end
	end

	def file_hash
		@digest=' '
		if File.exist?(@local_file) && File.readable?(@local_file)
			@digest = @@SHA256.hexdigest(File::read(@local_file))
		end
	end

	def self::log
		@@log
	end


	attr_accessor :updated, :size, :position, :local_file 
	attr_reader :date, :url, :digest
end
#-- EOF --
