#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2014-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
=end
require 'singleton'
require_relative 'download'
require_relative 'logging'

=begin
	A singleton of this type manages the previously unfinished or 
	queued downloads.
=end
class DownloadHistory
	include Singleton	
	include Logging
	self.extend(Logging)
        @@log =	init_logger()
	@@history_base = 'rget_queue'

	private
	def dump()
		@@log.debug('dump')
		unless @hthread && @hthread.alive?
			@hthread = Thread.new do 
				begin
					File.delete(@history_file) if File.exist?(@history_file)
					File.open(@history_file, 'w') do |hf|
						Marshal.dump(@history, hf)
					end
				rescue Exception => ex
					@log.error 'cannot write history-file: ' << ex.message
				end
			end
			@hthread = nil
		end
	end
	public
	# is there a history?
	def empty?
		!@history || @history.empty?
	end
	# load an existing list of unfinished downloads
	def initialize
		@log = @@log
	        @log.level = $log_level	
		home = ENV['HOME']
		@history = nil
		@hthread = nil
		if home
			hist_dir = home.dup << File::SEPARATOR << ".#{File::basename($0) }"
			if !File.exist?(hist_dir)
				begin
				Dir.mkdir(hist_dir, 0700)
				rescue Exception => ex
					@log.error('Cannot create history in ' << hist_dir.dup << ': ' << ex.message)
					@log.error('Aborting')
					exit false
				end
			end
			@history_file = hist_dir.dup << File::SEPARATOR << @@history_base
			load_history
			@log.debug("history-file is in #{@history_file}")
		else
			@log.error('Cannot determine the current users home-directory')
			exit false
		end
	end

	def load_history
		if(File.exist?(@history_file) && !File.zero?(@history_file)  )
			File.open(@history_file, 'r') do |hf|
				@history = Marshal.load(hf)	
				@log.debug ('history is ' << @history.to_s)
			end
		end
	end

	def queued_URLs
		@history ? @history.keys : []
	end

	# find and keep for later reference a previous attempt to load the file
	# from url
	def previous_download(url)
		@history ? @history[url] : nil
	end

	def [](ele)
		@log.debug('checking for element ' << ele.to_s)
		prev_dl = nil
		if(@history && !@history.empty?)
			if(ele.respond_to?(:to_str))
				@history[ele]
			elsif(ele.respond_to?(:to_int))
				@log.debug('looking for queued item #' << (ele).to_s)
				a = @history.to_a
				if(ele <= a.size)
					a[ele - 1][1]
				else(a.size > 0)
					raise StandardError.new('Queued items range from 1 to ' << a.size.to_s << '!')
				end
			end
		end
	end

	# update te download-history for the file at url.
	def update(url, opts = {})
		begin
			@log.debug('update')
			@history ||= Hash.new
			# produces a lot of output 
			if ! @history.has_key?(url) 
				@history[url] = Download.new(url, opts[:local])
			elsif(opts[:local])
				@history[url].local_file = opts[:local]
			end

			@history[url].position = opts[:position] if opts[:position]
			@history[url].size = opts[:size] if opts[:size]
			@history[url].updated=DateTime.now
			@history[url].file_hash
			@log.debug('dumping history ' << @history.inspect)
			dump
		rescue Exception => ex
			@log.error('CANNOT update history : ' << ex.message)
			@log.error('history is ' << @history.to_s)
			raise ex
		end
		@log.debug('download is ' << @history[url].to_s)
	end

	def delete(url)
		@log.debug('deleting from queue : ' << @history[url].to_s)
		@history.delete(url)
		dump
		load_history
	end

	def length()
		return @history.length
	end

	attr_reader :history_file 
end
# EOF
