#encoding: UTF-8
=begin
/***************************************************************************
 *   ©2011-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

=begin
A module to facilitate frequently occuring checks on
file-system objects
=end
module File_Checking

  @@text_messages  = {
        :exist? => "does not exist!",
        :exist => "does not exist!",
        :readable? => "is not readable!",
        :readable => "is not readable!",
        :executable? => "is not executable!",
        :executable => "is not executable!",
        :writable? => "is not writable!",
        :writable => "is not writable!",
        :directory? => "is not a directory!",
        :directory => "is not a directory!",
        :file? => "is not a file!",
        :file => "is not a file!",
      }

      # Checks if the file with the name from the first
      # parameter has the properties, listed in the second. 
      # The messages parameter is an array of one or several
      # of :exist?, :readable?, :writable?, :directory?  or 
      # their string-representations, respectively. 
      # Returns nil in case of success, otherwise an 
      # informative message, describing the first negative
      # test-result.
      def file_check(file, *messages)
        File_Checking.file_check(file, *messages)
      end
     
      # Checks if the file with the name from the first
      # parameter has the properties, listed in the second. 
      # The messages parameter is an array of one or all
      # of :exist?, :readable?, :writable?, :directory? or
      # their string-representations, respectively. 
      # Returns nil in case of success, otherwise an 
      # informative message, describing the first negative
      # test-result.
      def self.file_check(file, *messages)
        msg = nil
	if(file && messages.respond_to?(:to_ary) && !messages.empty?)
		messages.each do |k|
			if(! k.to_s.end_with?('?'))	
				k = (k.to_s << '?').to_sym
			end
			@log.debug ('checking ' << k.to_s) if @log
			if(msg == nil && File.respond_to?(k) && ! File.send(k, file.to_s))
				msg = "#{file} #{@@text_messages[k.to_sym]}"
			end
		end
	end
	msg
      end
end

=begin
# example

include File_Checking
msg = file_check('some_file.txt', [:exist?, :readable?, 'writable'])
puts msg if msg
=end

#eof
