#!/usr/bin/env ruby
#encoding 'UTF-8'
=begin
/***************************************************************************
 *   ©2015-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
=end

require_relative '../version'
require_relative 'argparser'
require_relative 'network'
require_relative 'logging'
require_relative 'download_history'
require_relative 'user_input'
require_relative 'util'

=begin
	A download manager.
=end
class Rget
	self.extend(Logging)
	@@log = init_logger(STDOUT, Logger::INFO)

	# reads commandline-options, organizes, reads or updates the
	# download-queue, cuts and washes the vegetables, irons my shirts and
	# starts the download. 
	def initialize()	
		@log = @@log
		options = ArgParser.parse(ARGV)
		@log.level = $log_level
		@log.debug('options are ' << options.to_s)

		# create the history only after the options have been read.
		$HISTORY = DownloadHistory.instance
		if(options.list_only)
			show_queue
			exit true
		end

		if(options.queued_item)
			@log.debug('queued item is ' << options.queued_item)
			dw = $HISTORY[options.queued_item]
			@log.debug('from queue: ' << dw.to_s)
			@url = $HISTORY[options.queued_item].url
			@file = $HISTORY[options.queued_item].local_file
		end

		@url ||= ARGV.first
		@file ||= ARGV.last if ARGV.length > 1
		if !@url || @url.strip.empty?
			@log.warn('No URL to download!')
			@log.info('Call with option "-h" to see some usage information')
			exit false
		else
			@log.debug('URL to download: ' << @url) 
			if(@file)
				@log.debug('Requested file to write: ' << @file) 
			else
				@file = File.basename(@url)
				@log.debug('Derived file to write: ' << @file) 
			end
			verify_file
		end
		@continue = options.continue
		
		#do it
		download
	end

	private

	def verify_file
		if(@file)
			if( File.exist?(@file))
				if(!File.writable?(@file)) 
					@log.error('The local file ' << @file.dup << ' exists and cannot be overwritten! Aborting!')
					exit false
				else
					puts('The local file ' << @file.dup << ' exists! Do you want to replace it or append data to this file (Y/n)?')	
					resp = wait_for_user	
					if(resp.chr.downcase != 'y')
						@log.info('Okay doing nothing')
						exit true
					else
						@log.debug('Local file will be modified!')
					end
				end
			elsif(File.exist?(File.dirname(@file)))
				if(!File.directory?(File.dirname(@file)) || !File.writable?(File.dirname(@file)))
					@log.error('Cannot create the file ' << @file.dup << ' in the directory ' << File.dirname(@file))
					@log.error('Aborting')
					exit false
				else
					@log.info('Will download to ' << @file)	
				end
			end
		else  # this should never happen.
			@log.error('No local file-name given. Aborting.')
			exit false
		end
	end
	
	# does it.
	def download()
		local = File.absolute_path(@file)

		network_opts = {}
		if(@url)
			check_history(@url) 
			if(@previous_download && @previous_download.local_file && File::exist?(@previous_download.local_file) )
				local = @previous_download.local_file
				network_opts[:previous_size] = @previous_download.size
			else
				$HISTORY.update(@url, :local=>local )
			end
			puts 'Will download ' << @url
			network_opts[:continue] = @continue

			Network::http_to_file(local, @url, network_opts)
		end
	end

=begin
    See if there is already an entry for the url in the history. Return
    the corresponding Download-object, if the user desires to continue
    downloading to the existing, local file, else nil.
=end
	def check_history(element)
		@previous_download = $HISTORY[element]
		if(@previous_download && @previous_download.local_file && File::exist?(@previous_download.local_file) )
			puts "\nQueued item found: " << @previous_download.url
			puts "A previous attempt to download this file has been interrupted."
			puts @previous_download.date.to_s << " local file: " << @previous_download.local_file << " current size: " << byte_units(File.size(@previous_download.local_file)) << ' bytes'
			puts "Do you want to continue downloading to " << @previous_download.local_file << " (Y/n)?"
			resp = wait_for_user
			if("\u0003" == resp)
				puts ("\n\tProgram interrupted by user (CTRL+C)")
				exit 0
			end
			unless ('y' == resp.chr.downcase)
				puts "\n\tIgnoring previous download. Will fetch the complete file."
				@previous_download = nil
			end
		end
	end

	def queued_url(element)
		$HISTORY[element].url
	end


=begin
	Print out the current list of unfinished downloads.
=end
	def show_queue()
		if($HISTORY.empty?)
			puts "There are no queued downloads."
		else
			puts "Queued downloads (%i):" %$HISTORY.queued_URLs.size
			$HISTORY.queued_URLs.each_with_index do |url, i|
				download = $HISTORY.previous_download(url)
				puts "\t%i) %s" %[i+1, url]
				puts "\t\t        since: %s" %download.date.strftime('%d.%m.%Y')
				puts "\t\t last attempt: %s" %download.updated.strftime('%d.%m.%Y %kh%M')
				puts "\t\texpected size: %s" %byte_units(download.size)
				puts "\t\t current size: %s" %byte_units(download.position)
				puts "\t\t   local file: %s" %download.local_file
				puts "\t---------------------"
			end
		end
	end
end
# --------------- EOF ---------
